/*
 * IRC - Internet Relay Chat, ircd/m_sajoin.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_sajoin.c,v 1.1 2003/11/23 19:00:00 styx Exp $
 */

#include "channel.h"
#include "config.h"
#include "client.h"
#include "handlers.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_chattr.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_relay.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_snprintf.h"
#include "ircd_alloc.h"
#include "list.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "querycmds.h"
#include "send.h"
#include "s_conf.h"
#include "s_user.h"
#include "s_debug.h"
#include "userload.h"
#include "patchlevel.h"

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

static int do_sajoin(struct Client* sptr, struct Client* acptr, char* channel)
{
//	struct Client *acptr;
	struct Channel *chptr;
	struct JoinBuf join;
	struct JoinBuf create;
	unsigned int flags = 0;
	int j;
	int k = 0;
	char *name;

	if (IsChannelService(acptr))
		return send_reply(sptr, ERR_NOPRIVILEGES);

	if (!FindChannel(channel)) {
		chptr = get_channel(acptr, channel, CGT_CREATE);
	} else {
		chptr = get_channel(acptr, channel, CGT_NO_CREATE);
	}

	if (find_member_link(chptr, acptr))
		return 0;

	joinbuf_init(&join, acptr, acptr, JOINBUF_TYPE_JOIN, 0, 0);
	joinbuf_init(&create, acptr, acptr, JOINBUF_TYPE_CREATE, 0, TStime());

	name = chptr->chname;
	clean_channelname(name);

	if (!IsChannelName(name)) {
		send_reply(sptr, ERR_NOSUCHCHANNEL, name);
		return 0;
	}

		for (k = 0, j = 0; name[j]; j++)
			if (IsCntrl(name[j]))
				k++;

		if ( k > 0 ) {
			send_reply(sptr, ERR_NOSUCHCHANNEL, name);
			return 0;
		}

		if (chptr->users == 0) {
			flags = CHFL_CHANOP;
		} else {
			flags = CHFL_DEOPPED;
		}

/* Est-ce vraiment utile ? - StyX
		if (cli_user(acptr)->joined >= feature_int(FEAT_MAXCHANNELSPERUSER)) {
			send_reply(sptr, ERR_TOOMANYCHANNELS, chptr ? chptr->chname : name);
			return 0;
		}
*/

		if (chptr) {
			joinbuf_join(&join, chptr, flags);
		}
		else if (!(chptr = get_channel(acptr, name, CGT_CREATE)))
			return 0;
		else if (check_target_limit(acptr, chptr, chptr->chname, 1)) {
			sub1_from_channel(chptr);
			return 0;
		}
		else
			joinbuf_join(&create, chptr, flags);

		if (chptr->topic[0]) {
			send_reply(acptr, RPL_TOPIC, chptr->chname, chptr->topic);
			send_reply(acptr, RPL_TOPICWHOTIME, chptr->chname, chptr->topic_nick, chptr->topic_time);
		}

		do_names(acptr, chptr, NAMES_ALL|NAMES_EON);

		joinbuf_flush(&join);
		joinbuf_flush(&create);

		sendto_opmask_butone(0, SNO_OLDSNO, "%s force %s  rejoindre le salon %s", cli_name(sptr), cli_name(acptr), chptr->chname);
		return 0;
}

/*
 *
 * mo_sajoin - Oprateurs
 * parv[0] = Expditeur
 * parv[1] = Pseudo
 * parv[2] = Salon
 *
 * -- StyX
 *
 */
int mo_sajoin(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
	struct Client *acptr;

	if (parc < 3)
		return need_more_params(sptr, "SAJOIN");

	if(!(acptr = FindClient(parv[1])))
		return send_reply(sptr, ERR_NOSUCHNICK, parv[1]);
	acptr = FindUser(parv[1]);

	return do_sajoin(sptr, acptr, parv[2]);
}

/*
 *
 * ms_sajoin - Serveurs
 * parv[0] = Expditeur
 * parv[1] = Numrique
 * parv[2] = Salon
 *
 * -- StyX
 *
 */
int ms_sajoin(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
	struct Client *acptr;

	if (parc < 3) {
		protocol_violation(sptr,"Too few arguments for SAJOIN");
		return need_more_params(sptr, "SAJOIN");
	}

	if ((acptr = findNUser(parv[1])) && !IsUser(acptr)) {
		protocol_violation(sptr,"Invalid numeric for SAJOIN");
		return 0;
	}

	return do_sajoin(sptr, acptr, parv[2]);
}
